/*
 *  ThemeEndianness.c
 *  ShapeShifter
 *
 *  Created by Jason Harris on 2/11/06.
 *  Copyright 2006 Geekspiff. All rights reserved.
 *
 */

#include "ThemeEndianness.h"
#include <Carbon/Carbon.h>

typedef struct {
    UInt32		version;            // always set to zero
    UInt32		isLittleEndian;		// if zero, all data in file is big-endian.  otherwise, all data is little-endian
} ThemeEndianness;


ThemeDataError 
ThemeDataFormatForPath(CFStringRef inPath, 
                       ThemeDataFormat *outFormat)
{
    if ( ! inPath || ! outFormat ) return kThemeDataBadParameter;
    
    CFURLRef    url = NULL;
    CFStringRef pathExtension = NULL;
    CFStringRef filename = NULL;
    
    url = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, inPath, kCFURLPOSIXPathStyle, FALSE);
	if( ! url ) return kThemeDataBadParameter;

    FSRef fsRef;
    if( CFURLGetFSRef(url, &fsRef) )
    {
        Boolean targetIsFolder, wasAliased;
        if ( (FSResolveAliasFile (&fsRef, true, &targetIsFolder, &wasAliased) == noErr) && wasAliased )
        {
            CFRelease(url);
            url = CFURLCreateFromFSRef(kCFAllocatorDefault, &fsRef);
        }
	}
    if ( ! url ) return kThemeDataBadParameter;

    pathExtension = CFURLCopyPathExtension(url);
    CFURLRef _url = CFURLCreateCopyDeletingPathExtension(kCFAllocatorDefault, url);
    if ( _url )
    {
        CFRelease(url);
        url = _url;
    }
    if ( ! url ) return kThemeDataBadParameter;
    
    filename = CFURLCopyLastPathComponent(url);
    
    *outFormat = kThemeDataFormatUnknown;
    if ( pathExtension && filename )
    {
        if ( kCFCompareEqualTo == CFStringCompare( pathExtension, CFSTR("rsrc"), 0) )
        {
            if ( kCFCompareEqualTo == CFStringCompare( filename, CFSTR("Extras2"), 0) )
                *outFormat = kThemeDataLittleEndian;
        }
    }
    
    CFRelease(url);
    if ( pathExtension) CFRelease(pathExtension);
    if ( filename) CFRelease(filename);
    return kThemeDataNoError;
}


ThemeDataError 
ThemeDataFormatForResourceFile(SInt16 inRefNum, 
                               ThemeDataFormat *outFormat)
{
    SInt16          oldRefNum;
    Handle          theResource;
    ThemeDataError  result = kThemeDataNoError;
    
    if ( -1 == inRefNum || ! outFormat ) return kThemeDataBadParameter;
    
    oldRefNum = CurResFile();
    UseResFile(inRefNum);
    theResource = Get1Resource('Endn', 128);
    if ( theResource )
    {
        HLock(theResource);
        ThemeEndianness *endn = (ThemeEndianness *)*theResource;
        endn->version = CFSwapInt32BigToHost(endn->version);
        endn->isLittleEndian = CFSwapInt32BigToHost(endn->isLittleEndian);
        if ( 0 == endn->version )
        {
            if ( endn->isLittleEndian )
                *outFormat = kThemeDataLittleEndian;
            else
                *outFormat = kThemeDataBigEndian;
        }
        else
        {
            *outFormat = kThemeDataFormatUnknown;
            result = kThemeDataBadResource;
        }
        ReleaseResource(theResource);
    }
    else
        *outFormat = kThemeDataBigEndian;
    
    UseResFile(oldRefNum);
    return result;
}


ThemeDataError 
SetThemeDataFormatForResourceFile(SInt16 inRefNum, 
                                  ThemeDataFormat inFormat)
{
    SInt16          oldRefNum;
    Handle          endnHandle, newResource = NULL;
    ThemeDataError  result = kThemeDataUnexpectedError;
    
    if ( -1 == inRefNum || kThemeDataFormatUnknown == inFormat ) return kThemeDataBadParameter;
    
    oldRefNum = CurResFile();
    UseResFile(inRefNum);
    
    endnHandle = NewHandle(sizeof(ThemeEndianness));
    if ( ! endnHandle ) goto DONE;
    HLock(endnHandle);
    ThemeEndianness *endn = (ThemeEndianness *)*endnHandle;
    endn->version = CFSwapInt32HostToBig(0);
    endn->isLittleEndian = CFSwapInt32HostToBig((kThemeDataLittleEndian == inFormat) ? 1 : 0);
    
    // if this resource is already in the file, delete it
    SetResLoad(false);
    result = kThemeDataBadResource;
    Handle oldResource = Get1Resource('Endn', 128);
    if ( oldResource )
    {
        RemoveResource(oldResource);
        if ( noErr != ResError() )
            goto DONE;
        DisposeHandle(oldResource);
    }
    SetResLoad(true);
    
    // add in the new resource data
    AddResource( endnHandle, 'Endn', 128, "\p" );
    if( noErr != ResError() ) goto DONE;
    newResource = endnHandle;
    endnHandle = NULL;
    ChangedResource(newResource);
    if( noErr != ResError() ) goto DONE;
    WriteResource(newResource);
    if( noErr != ResError() ) goto DONE;
    result = kThemeDataNoError;
    
DONE:
    if ( endnHandle )
        DisposeHandle(endnHandle);
    if ( newResource )
        ReleaseResource(newResource);
    UseResFile(oldRefNum);
    return result;
}
